import io from 'socket.io-client'
import xmlrpc from 'homematic-xmlrpc'
import {
  CallGetChannels, CallUpdateDevice,
  CallUpdateDevices,
  intervalCheckUpdate,
  IsOnlineInterval,
  SetDeviceIntervalUpdate, SetDeviceIntervalUpdateTime, SetIsOnlineIntervalTime,
  UpdateDevicesInterval
} from './CcuCalls';
import { clientRpcJsonIismSerial, clientRpcJsonIismService, hostSocket } from './Config';


const clientRpc = xmlrpc.createClient({
  host: '127.0.0.1',
  port: 2001,
  path: "/"
})
let session = null
let serial = null

let socket = io(
  hostSocket,
  { transports: ["websocket"], rejectUnauthorized: false, secure: true, reconnection: true }
);

socket.on("connect", async (e) => {
  console.log("connect")

  let resSerial = await clientRpcJsonIismSerial.call(
    {
      "jsonrpc": "2.0",
      version: "1.1",
      "method": "CCU.getSerial",
      "params": { "_session_id_": session },
      "id": 1
    }
  );
  serial = resSerial.serial
  console.log("serial", serial)
  socket.emit("ccu/connectCcu", serial)
  IsOnlineInterval(socket, serial);
  await CallGetChannels(socket, serial);
  UpdateDevicesInterval(socket, serial);

  socket.on('intervalCheckUpdate', (data) => {
    SetDeviceIntervalUpdate(data)
  })

  socket.on('intervalCheckUpdateTime', (data) => {
    SetDeviceIntervalUpdateTime(data)
    UpdateDevicesInterval(socket, serial);
  })

  socket.on('intervalIsOnlineTime', (data) => {
    SetIsOnlineIntervalTime(data)
    IsOnlineInterval(socket, serial);
  })

  socket.on('isOnline', () => {
    socket.emit("ccu/isOnline", serial)
  })
  socket.on('callRpcJson', async (data) => {
    console.log("callRpcJson")
    data.params["_session_id_"] = session;
    let resCallRpcJson = await clientRpcJsonIismService.call(
      {
        "jsonrpc": "2.0",
        version: "1.1",
        "method": data.method,
        "params": data.params,
        "id": data.id,
      });

    socket.emit("ccu/callRpcJsonResponse", resCallRpcJson)
  })

  socket.on('callRpcXml', (data) => {
    console.log("callRpcXml")
    clientRpc.methodCall(data.method, data.params, function (error, dataResponse) {
      socket.emit("ccu/callRpcXmlResponse", { id: data.id, result: dataResponse })
    });
  })

  socket.on('getDeviceValue', async (data) => {
    console.log("getDeviceValue")
    let { address, valueKey, id } = data
    let resDeviceValue = await clientRpcJsonIismService.call(
      {
        "jsonrpc": "2.0",
        version: "1.1",
        "method": "Interface.getValue",
        "params": {
          "_session_id_": session, address,
          "interface": data.interface, valueKey
        },
        id
      });
    socket.emit("ccu/getDeviceValueResponse", resDeviceValue)
  })
  socket.on('setDeviceValue', async (data) => {
    let { address, valueKey, type, value, id } = data
    let resDeviceValue = await clientRpcJsonIismService.call(
      {
        "jsonrpc": "2.0",
        version: "1.1",
        "method": "Interface.setValue",
        "params": {
          "_session_id_": session, address,
          "interface": data.interface, valueKey, type, value
        },
        id
      }
    );
    await CallUpdateDevice(data.channelId, socket, serial)
    socket.emit("ccu/setDeviceValueResponse", { resDeviceValue, data })
  })
  socket.on('setChannelName', async (data) => {
    console.log("setChannelName")
    let { id, name } = data
    let resDeviceValue = await clientRpcJsonIismService.call(
      {
        "jsonrpc": "2.0",
        version: "1.1",
        "method": "Channel.setName",
        "params": {
          "_session_id_": session, id, name
        },
        id
      }
    );
    socket.emit("ccu/setChannelNameResponse", { resDeviceValue, data })
  })
  socket.on('setDeviceName', async (data) => {
    console.log("setDeviceName")
    let { id, name } = data
    let resDeviceValue = await clientRpcJsonIismService.call(
      {
        "jsonrpc": "2.0",
        version: "1.1",
        "method": "Device.setName",
        "params": {
          "_session_id_": session, id, name
        },
        id
      }
    );
    socket.emit("ccu/setChannelNameResponse", { resDeviceValue, data })
  })

  socket.on('getRooms', async () => {
    console.log("getRooms")
    let roomsRpc = await clientRpcJsonIismService.call(
      {
        "jsonrpc": "2.0",
        version: "1.1",
        "method": "Room.getAll",
        "params": { "_session_id_": session },
        "id": 0
      });

    if (roomsRpc.result !== null) {
      console.log("getRooms end")
      socket.emit("ccu/setRooms", { result: roomsRpc.result, serial })
    }
  })
  socket.on('getDevices', async () => {
    console.log("getDevices")
    let devices = [];
    let devicesRpc = await clientRpcJsonIismService.call(
      {
        "jsonrpc": "2.0",
        version: "1.1",
        "method": "Device.listAll",
        "params": { "_session_id_": session },
        "id": 0
      });
    if (devicesRpc.result !== null) {
      for (let i = 0; i < devicesRpc.result.length; i++) {
        let deviceObject = devicesRpc.result[i]
        let device = await clientRpcJsonIismService.call(
          {
            "jsonrpc": "2.0",
            version: "1.1",
            "method": "Device.get",
            "params": { "_session_id_": session, "id": deviceObject },
            "id": 0
          });
        if (device.result !== null) {
          let deviceParameterMaster = await clientRpcJsonIismService.call(
            {
              "jsonrpc": "2.0",
              version: "1.1",
              "method": "Interface.getParamset",
              "params": {
                "_session_id_": session,
                "interface": device.result.interface,
                "address": device.result.address,
                "paramsetKey": "MASTER"
              },
              "id": 0
            });
          if (deviceParameterMaster.result !== null) {
            device.result.masterParams = deviceParameterMaster.result
          }
          let deviceParameterValue = await clientRpcJsonIismService.call(
            {
              "jsonrpc": "2.0",
              version: "1.1",
              "method": "Interface.getParamset",
              "params": {
                "_session_id_": session,
                "interface": device.result.interface,
                "address": device.result.address,
                "paramsetKey": "VALUE"
              },
              "id": 0
            });
          if (deviceParameterValue.result !== null) {
            device.result.valueParams = deviceParameterValue.result
          }

          for (let ci = 0; ci < device.result.channels.length; ci++) {
            let channel = device.result.channels[ci]

            let channelParameterMaster = await clientRpcJsonIismService.call(
              {
                "jsonrpc": "2.0",
                version: "1.1",
                "method": "Interface.getParamsetDescription",
                "params": {
                  "_session_id_": session,
                  "interface": device.result.interface,
                  "address": channel.address,
                  "paramsetKey": "MASTER"
                },
                "id": 0
              });
            if (channelParameterMaster.result !== null) {
              device.result.channels[ci].masterParams = channelParameterMaster.result
              /*for (let cmi=0; cmi < device.result.channels[ci].masterParams.length; cmi++) {
                let paramMaster = device.result.channels[ci].masterParams[cmi];
                let parameterMasterValue = await clientRpcJson.call(
                  {
                    "jsonrpc": "2.0",
                    version: "1.1",
                    "method": "Interface.getMasterValue",
                    "params": { "_session_id_": session, "interface": device.result.interface, "address": channel.address, "valueKey": paramMaster.NAME },
                    "id": 0
                  });
                if (parameterMasterValue.result !== null) {
                  device.result.channels[ci].masterParams[cmi].value = parameterMasterValue.result
                }
              }*/
            }
            let channelParameterValue = await clientRpcJsonIismService.call(
              {
                "jsonrpc": "2.0",
                version: "1.1",
                "method": "Interface.getParamsetDescription",
                "params": {
                  "_session_id_": session,
                  "interface": device.result.interface,
                  "address": channel.address,
                  "paramsetKey": "VALUE"
                },
                "id": 0
              });
            if (channelParameterValue.result !== null) {
              device.result.channels[ci].valueParams = channelParameterValue.result
              /*for (let cvi=0; cvi < device.result.channels[ci].valueParams.length; cvi++) {
                let paramMaster = device.result.channels[ci].valueParams[cvi];
                let parameterMasterValue = await clientRpcJson.call(
                  {
                    "jsonrpc": "2.0",
                    version: "1.1",
                    "method": "Interface.getMasterValue",
                    "params": { "_session_id_": session, "interface": device.result.interface, "address": channel.address, "valueKey": paramMaster.NAME },
                    "id": 0
                  });
                if (parameterMasterValue.result !== null) {
                  device.result.channels[ci].valueParams[cvi].value = parameterMasterValue.result
                }
              }*/
            }
          }

        }
        devices.push(device.result)
        if (i === devicesRpc.result.length - 1) {
          socket.emit("ccu/setDevices", { devices, serial })
        }
      }
    }
    console.log("getDevices end")
  });
});


/*
clientRpc.methodCall('listBidcosInterfaces', [], function (error, devices) {

  //console.log(devices)
})
clientRpc.methodCall('init', [], function (error, devices) {
  clientRpc.methodCall('listDevices', [], function (error, devices) {
    //console.log(devices)
    for (let i = 0; i < devices.length; i++) {
      let device = devices[i]
      clientRpc.methodCall('getDeviceDescription', [device.ADDRESS], function (error, device) {
        // console.log(device)
      })
    }
  })
})*/

/*clientRpcJson.call(
  {
    "jsonrpc": "2.0",
    "method": "Interface.getDeviceDescription",
    "params": { "_session_id_": session, "address": "0015DBE99EB2CD:1",
      "interface": "HmIP-RF" },
    "id": 1
  },
  function (err, res) {
    serial = res.result
    console.log("getDeviceDescription", res)
  }
);
clientRpcJson.call(
  {
    "jsonrpc": "2.0",
    "method": "Interface.getParamsetDescription",
    "params": { "_session_id_": session, "address": "0015DBE99EB2CD:2",
      "interface": "HmIP-RF", paramsetKey: "VALUES" },
    "id": 1
  },
  function (err, res) {
    serial = res.result
    console.log("getParamsetDescription", res)
  }
);
*/
/*clientRpcJson.call(
        {
          "jsonrpc": "2.0",
          "method": "Interface.setValue",
          "params": { "_session_id_": session, "address": "0015DBE99EB2CD:2",
            "interface": "HmIP-RF", valueKey: "STATE", type: 'bool', value: "0"  },
          "id": 1
        },
        function (err, res) {
          serial = res.result
          console.log("setValue", res)
        }
      );*/
